@@ -23,7 +23,17 @@
 #' For a \code{logical} or \code{factor} with only two levels, the equation above is
 #' \eqn{\bar{x}_g = \hat{p}_g}, i.e. the sample proportion and \eqn{s^2_g = \hat{p}_g(1 - \hat{p}_g)}.
 #'
+#' When using the SMD to evaluate the effectiveness of weighting in achieving
+#' covariate balance, it is important to isolate the change in SMD before and
+#' after weighting to the change in mean difference, so the denominator (covariance matrix)
+#' must be held constant (Stuart 2008, \doi{doi:10.1002/sim.3207}).
+#' By default, the unweighted covariance matrix is used to compute SMD in both
+#' the unweighted and weighted case. If the weights are not being used to adjust
+#' for covariate imbalance (e.g. case weights), the \code{unwgt.var} argument
+#' can be set to \code{FALSE} to use the weighted covariance matrix as the denominator.
+#'
 #' @name smd
+#'
 #' @param x a \code{vector} or \code{matrix} of values
 #' @param g a vector of at least 2 groups to compare. This should coercable to a
 #'    \code{factor}.
@@ -33,6 +43,7 @@
 #' @param na.rm Remove \code{NA} values from \code{x}? Defaults to \code{FALSE}.
 #' @param gref an integer indicating which level of \code{g} to use as the reference
 #'     group. Defaults to \code{1}.
+#' @param unwgt.var Use unweighted or weighted covariance matrix. Defaults to \code{TRUE}
 #' @importFrom methods setGeneric setMethod
 #' @return a \code{data.frame} containing standardized mean differences between
 #'    levels of \code{g} for values of \code{x}. The \code{data.frame} contains
@@ -49,17 +60,16 @@
 #' smd(x, g)
 setGeneric(
   "smd",
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-
-    if(gref < 1 || gref > length(unique(g))){
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
+    if (gref < 1 || gref > length(unique(g))) {
       stop(sprintf("gref must be an integer within %s", length(unique(g))))
     }
 
-    parts <- compute_smd_parts(.x = x, .g = g, .w = w, .na = na.rm, .ref = gref)
-    d     <- compute_smd_pairwise(parts)
-    out   <- list(term = names(d), estimate = unname(d))
+    parts <- compute_smd_parts(.x = x, .g = g, .w = w, .na = na.rm, .ref = gref, .unwgt = unwgt.var)
+    d <- compute_smd_pairwise(parts)
+    out <- list(term = names(d), estimate = unname(d))
 
-    if(std.error){
+    if (std.error) {
       ste <- unlist(Map(compute_smd_var, d = d, smd_parts = parts))
       out <- append(out, list(std.error = unname(sqrt(ste))))
     }
@@ -74,8 +84,8 @@ setGeneric(
 setMethod(
   "smd",
   signature = c("character", "ANY", "missing"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-    smd(x = as.factor(x), g = g, std.error = std.error, na.rm = na.rm, gref = gref)
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
+    smd(x = as.factor(x), g = g, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
   }
 )
 
@@ -85,8 +95,8 @@ setMethod(
 setMethod(
   "smd",
   signature = c("character", "ANY", "numeric"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-    smd(x = as.factor(x), g = g, w = w, std.error = std.error, na.rm = na.rm, gref = gref)
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
+    smd(x = as.factor(x), g = g, w = w, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
   }
 )
 
@@ -96,8 +106,8 @@ setMethod(
 setMethod(
   "smd",
   signature = c("logical", "ANY", "missing"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-    smd(x = as.numeric(x), g = g, std.error = std.error, na.rm = na.rm, gref = gref)
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
+    smd(x = as.numeric(x), g = g, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
   }
 )
 
@@ -107,8 +117,8 @@ setMethod(
 setMethod(
   "smd",
   signature = c("logical", "ANY", "numeric"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-    smd(x = as.numeric(x), g = g, w = w, std.error = std.error, na.rm = na.rm, gref = gref)
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
+    smd(x = as.numeric(x), g = g, w = w, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
   }
 )
 
@@ -117,15 +127,14 @@ setMethod(
 
 setMethod(
   "smd",
-  signature = c("matrix", "ANY",  "missing"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-
-    if(std.error){
+  signature = c("matrix", "ANY", "missing"),
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
+    if (std.error) {
       stop("smd is not set up to compute std.error on a matrix")
     }
 
     apply(x, 2, function(j) {
-      simplify2array(smd(x = j, g = g, std.error = std.error, na.rm = na.rm, gref = gref)$estimate)
+      simplify2array(smd(x = j, g = g, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)$estimate)
     })
   }
 )
@@ -136,15 +145,18 @@ setMethod(
 setMethod(
   "smd",
   signature = c("matrix", "ANY", "numeric"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-    if(std.error){
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
+    if (std.error) {
       stop("smd is not set up to compute std.error on a matrix")
     }
 
     apply(x, 2, function(j) {
-      simplify2array(smd(x = j, w = w, g = g,
-                         std.error = std.error,
-                         na.rm = na.rm, gref = gref)$estimate)
+      simplify2array(smd(
+        x = j, w = w, g = g,
+        std.error = std.error,
+        na.rm = na.rm, gref = gref,
+        unwgt.var = unwgt.var
+      )$estimate)
     })
   }
 )
@@ -155,10 +167,9 @@ setMethod(
 setMethod(
   "smd",
   signature = c("list", "ANY", "missing"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
     tidy_smd_multiplevar(lapply(x, function(z) {
-      smd(x = z, g = g, std.error = std.error, na.rm = na.rm, gref = gref)
+      smd(x = z, g = g, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
     }))
   }
 )
@@ -168,11 +179,10 @@ setMethod(
 
 setMethod(
   "smd",
-  signature = c("list",  "ANY", "numeric"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-
+  signature = c("list", "ANY", "numeric"),
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
     tidy_smd_multiplevar(lapply(x, function(z) {
-      smd(x = z, g = g, w = w, std.error = std.error, na.rm = na.rm, gref = gref)
+      smd(x = z, g = g, w = w, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
     }))
   }
 )
@@ -183,10 +193,9 @@ setMethod(
 setMethod(
   "smd",
   signature = c("data.frame", "ANY", "missing"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
     tidy_smd_multiplevar(lapply(x, function(z) {
-      smd(x = z, g = g, std.error = std.error, na.rm = na.rm, gref = gref)
+      smd(x = z, g = g, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
     }))
   }
 )
@@ -197,10 +206,9 @@ setMethod(
 setMethod(
   "smd",
   signature = c("data.frame", "ANY", "numeric"),
-  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L){
-
+  def = function(x, g, w, std.error = FALSE, na.rm = FALSE, gref = 1L, unwgt.var = TRUE) {
     tidy_smd_multiplevar(lapply(x, function(z) {
-      smd(x = z, g = g,  w = w, std.error = std.error, na.rm = na.rm, gref = gref)
+      smd(x = z, g = g, w = w, std.error = std.error, na.rm = na.rm, gref = gref, unwgt.var = unwgt.var)
     }))
   }
 )
@@ -247,15 +255,15 @@ setMethod(
 #' @seealso \code{\link{smd}}
 #' @keywords internal
 
-compute_smd_pairwise <- function(smd_parts){
+compute_smd_pairwise <- function(smd_parts) {
   simplify2array(lapply(smd_parts, function(x) compute_smd(x$D, x$S)))
 }
 
 #' @rdname compute_smd
 
-compute_smd <- function(D, S){
+compute_smd <- function(D, S) {
   out <- sqrt(t(D) %*% (MASS::ginv(S) %*% D))
-  if(length(D) == 1){
+  if (length(D) == 1) {
     out <- drop(out) * sign(D)
   }
   out
@@ -273,10 +281,10 @@ compute_smd <- function(D, S){
 #' @param d an SMD value
 #' @inheritParams compute_smd
 #' @keywords internal
-compute_smd_var <- function(d, smd_parts){
+compute_smd_var <- function(d, smd_parts) {
   N <- smd_parts$N
   sn <- sum(N)
-  sn/prod(N) + (d^2)/(2*sn)
+  sn / prod(N) + (d^2) / (2 * sn)
 }
 
 #' Compute components of SMD
@@ -288,24 +296,24 @@ compute_smd_var <- function(d, smd_parts){
 #' @param .g a vector of groupings to compare
 #' @param .na \code{TRUE/FALSE}. \code{NA} handling
 #' @param .ref integer position of the reference group
+#' @param .unwgt Use unweighted or weighted covariance.
 #' @param applyFUN the \code{FUN} used to compute the SMD parts. Defaults to
 #' \code{\link{n_mean_var}}
 #' @keywords internal
-compute_smd_parts <- function(.x, .g, .w, .na, .ref,
-                              applyFUN = n_mean_var){
-
+compute_smd_parts <- function(.x, .g, .w, .na, .ref, .unwgt,
+                              applyFUN = n_mean_var) {
   # Checks
-  if(length(.x) != length(.g)){
+  if (length(.x) != length(.g)) {
     stop("Length of x and g must match")
   }
 
-  if(anyNA(.x) && !.na){
+  if (anyNA(.x) && !.na) {
     stop("x contains NA values. Do you need to set na.rm = TRUE?")
   }
 
-  ng  <- length(unique(.g))
+  ng <- length(unique(.g))
 
-  if(ng < 2){
+  if (ng < 2) {
     stop("g must contain at least two levels.")
   }
 
@@ -316,17 +324,17 @@ compute_smd_parts <- function(.x, .g, .w, .na, .ref,
   # * the vectors x and w may be different types so that can't be cbind()ed
   #   before splitting without unintended coersion.
   # So my current tack is to combine them into a data.frame first.
-  dd  <- data.frame(x = .x)
-  dd$w <- if(missing(.w)) NULL else .w
-  ll  <- split.data.frame(dd, f = .g)
-  U   <- simplify2array(lapply(ll, function(M){
-    do.call(applyFUN, args = append(M, list(na.rm = .na)))
+  dd <- data.frame(x = .x)
+  dd$w <- if (missing(.w)) NULL else .w
+  ll <- split.data.frame(dd, f = .g)
+  U <- simplify2array(lapply(ll, function(M) {
+    do.call(applyFUN, args = append(M, list(na.rm = .na, unwgt.var = .unwgt)))
   }))
 
   # Create pairwise components
   N <- lapplyFUNpairwise(U["n", ], c, .ref)
   D <- lapplyFUNpairwise(U["mean", ], `-`, .ref)
-  S <- lapplyFUNpairwise(U["var", ], function(x, y) (x + y)/2, .ref)
+  S <- lapplyFUNpairwise(U["var", ], function(x, y) (x + y) / 2, .ref)
 
   Map(list, N = N, D = D, S = S)
 }
@@ -337,20 +345,22 @@ compute_smd_parts <- function(.x, .g, .w, .na, .ref,
 #' @param smd_res a result of \link{smd}
 #' @return a \code{data.frame}
 #' @keywords internal
-tidy_smd_singlevar <- function(smd_res){
+tidy_smd_singlevar <- function(smd_res) {
   data.frame(smd_res, stringsAsFactors = FALSE)
 }
 
 #' @rdname smd_tidier
-tidy_smd_multiplevar <- function(smd_res){
-  if(length(names(smd_res)) == length(smd_res)){
-    hold <- lapply(seq_along(smd_res), function(i){
-      data.frame(variable = names(smd_res)[i], smd_res[[i]],
-                 stringsAsFactors = FALSE)
+tidy_smd_multiplevar <- function(smd_res) {
+  if (length(names(smd_res)) == length(smd_res)) {
+    hold <- lapply(seq_along(smd_res), function(i) {
+      data.frame(
+        variable = names(smd_res)[i], smd_res[[i]],
+        stringsAsFactors = FALSE
+      )
     })
   } else {
-    hold <- lapply(smd_res, function(x){
-      data.frame(x,stringsAsFactors = FALSE)
+    hold <- lapply(smd_res, function(x) {
+      data.frame(x, stringsAsFactors = FALSE)
     })
   }
 
